@@ -16,13 +16,20 @@
 # Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301, USA, or see http://www.gnu.org/.
 
-VERSION = "2.6"
+VERSION = "2.9"
 STATE_VERSION = 2
 import feedparser, feedfinder, plugins
 from persister import Persistable, Persister
 import os, time, sha, getopt, sys, re, cgi, socket, urllib2, calendar
-import string, threading
+import string, locale
 from StringIO import StringIO
+import types
+
+try:
+	import threading
+	have_threading = 1
+except:
+	have_threading = 0
 
 def set_socket_timeout(n):
 	"""Set the system socket timeout."""
@@ -69,8 +76,8 @@ def sanitise_html(html, baseurl, inline, config):
 	# sgmllib handles "<br/>/" as a SHORTTAG; this workaround from
 	# feedparser.
 	html = re.sub(r'(\S)/>', r'\1 />', html)
-	html = feedparser._resolveRelativeURIs(html, baseurl, None)
-	p = feedparser._HTMLSanitizer(None)
+	html = feedparser._resolveRelativeURIs(html, baseurl, "UTF-8")
+	p = feedparser._HTMLSanitizer("UTF-8")
 	p.feed(html)
 	html = p.output()
 
@@ -85,14 +92,14 @@ def sanitise_html(html, baseurl, inline, config):
 
 	if config["tidyhtml"]:
 		import mx.Tidy
-		# mx.Tidy won't accept Unicode strings, so we have to
-		# encode data as UTF-8 for it.
-		output = mx.Tidy.tidy(html.encode("UTF-8"), None, None,
-		                      wrap = 0, numeric_entities = 1)[2]
-		output = output.decode("UTF-8")
+		args = { "wrap": 0, "numeric_entities": 1 }
+		plugins.call_hook("mxtidy_args", config, args, baseurl, inline)
+		output = mx.Tidy.tidy(html, None, None,
+		                      **args)[2]
 		html = output[output.find("<body>") + 6
 		              : output.rfind("</body>")].strip()
 
+	html = html.decode("UTF-8")
 	box = plugins.Box(html)
 	plugins.call_hook("clean_html", config, box, baseurl, inline)
 	return box.value
@@ -151,11 +158,19 @@ def author_to_html(entry, feedurl, config):
 		name = entry.get("author")
 
 	url = None
+	fallback = "author"
 	if author_detail is not None:
 		if author_detail.has_key("url"):
 			url = author_detail["url"]
 		elif author_detail.has_key("email"):
 			url = "mailto:" + author_detail["email"]
+		if author_detail.has_key("email"):
+			fallback = author_detail["email"]
+		elif author_detail.has_key("url"):
+			fallback = author_detail["url"]
+
+	if name == "":
+		name = fallback
 
 	if url is None:
 		html = name
@@ -198,7 +213,10 @@ def fill_template(template, bits):
 				if if_stack != []:
 					if_stack.append(not if_stack.pop())
 			elif bits.has_key(key):
-				write(bits[key])
+				if type(bits[key]) == types.UnicodeType:
+					write(bits[key].encode(locale.getpreferredencoding()))
+				else:
+					write(bits[key])
 		else:
 			write(part)
 	return f.getvalue()
@@ -271,20 +289,8 @@ class Feed:
 	def fetch(self, rawdog, config):
 		"""Fetch the current set of articles from the feed."""
 
-		class DummyPasswordMgr:
-			def __init__(self, creds):
-				self.creds = creds
-			def add_password(self, realm, uri, user, passwd):
-				pass
-			def find_user_password(self, realm, authuri):
-				return self.creds
-
 		handlers = []
 
-		if self.args.has_key("user") and self.args.has_key("password"):
-			mgr = DummyPasswordMgr((self.args["user"], self.args["password"]))
-			handlers.append(urllib2.HTTPBasicAuthHandler(mgr))
-
 		proxies = {}
 		for key, arg in self.args.items():
 			if key.endswith("_proxy"):
@@ -298,12 +304,22 @@ class Feed:
 
 		plugins.call_hook("add_urllib2_handlers", rawdog, config, self, handlers)
 
+		auth_creds = None
+		if self.args.has_key("user") and self.args.has_key("password"):
+			auth_creds = (self.args["user"], self.args["password"])
+
+		use_im = True
+		if self.get_keepmin(config) == 0 or config["currentonly"]:
+			use_im = False
+
 		try:
 			return feedparser.parse(self.url,
 				etag = self.etag,
 				modified = self.modified,
 				agent = "rawdog/" + VERSION,
-				handlers = handlers)
+				handlers = handlers,
+				auth_creds = auth_creds,
+				use_im = use_im)
 		except:
 			return None
 
@@ -425,6 +441,12 @@ class Feed:
 			r = self.get_html_name(config).lower()
 			return non_alphanumeric_re.sub('', r)
 
+	def get_keepmin(self, config):
+		try:
+			return int(self.args["keepmin"])
+		except:
+			return config["keepmin"]
+
 class Article:
 	"""An article retrieved from an RSS feed."""
 
@@ -566,7 +588,8 @@ class Config:
 
 	def __init__(self):
 		self.files_loaded = []
-		self.loglock = threading.Lock()
+		if have_threading:
+			self.loglock = threading.Lock()
 		self.reset()
 
 	def reset(self):
@@ -578,6 +601,7 @@ class Config:
 			"maxarticles" : 200,
 			"maxage" : 0,
 			"expireage" : 24 * 60 * 60,
+			"keepmin" : 0,
 			"dayformat" : "%A, %d %B %Y",
 			"timeformat" : "%I:%M %p",
 			"datetimeformat" : None,
@@ -672,6 +696,8 @@ class Config:
 			self["maxage"] = parse_time(l[1])
 		elif l[0] == "expireage":
 			self["expireage"] = parse_time(l[1])
+		elif l[0] == "keepmin":
+			self["keepmin"] = int(l[1])
 		elif l[0] == "dayformat":
 			self["dayformat"] = l[1]
 		elif l[0] == "timeformat":
@@ -727,9 +753,11 @@ class Config:
 	def log(self, *args):
 		"""If running in verbose mode, print a status message."""
 		if self["verbose"]:
-			self.loglock.acquire()
+			if have_threading:
+				self.loglock.acquire()
 			print >>sys.stderr, "".join(map(str, args))
-			self.loglock.release()
+			if have_threading:
+				self.loglock.release()
 
 	def bug(self, *args):
 		"""Report detection of a bug in rawdog."""
@@ -950,7 +978,7 @@ class Rawdog(Persistable):
 		numfeeds = len(update_feeds)
 		config.log("Will update ", numfeeds, " feeds")
 
-		if config["numthreads"] > 0:
+		if have_threading and config["numthreads"] > 0:
 			fetcher = FeedFetcher(self, update_feeds, config)
 			prefetched = fetcher.run(config["numthreads"])
 		else:
@@ -973,12 +1001,20 @@ class Rawdog(Persistable):
 			if rc:
 				seen_some_items[url] = 1
 
+		feedcounts = {}
+		for key, article in self.articles.items():
+			url = article.feed
+			feedcounts[url] = feedcounts.get(url, 0) + 1
+
 		count = 0
 		for key, article in self.articles.items():
-			if (seen_some_items.has_key(article.feed)
-			    and article.can_expire(now, config)):
+			url = article.feed
+			if (seen_some_items.has_key(url)
+			    and article.can_expire(now, config)
+			    and feedcounts[url] > self.feeds[url].get_keepmin(config)):
 				plugins.call_hook("article_expired", self, config, article, now)
 				count += 1
+				feedcounts[url] -= 1
 				del self.articles[key]
 		config.log("Expired ", count, " articles, leaving ", len(self.articles))
 
@@ -1197,7 +1233,7 @@ __description__
 		f = StringIO()
 		print >>f, """<table id="feeds">
 <tr id="feedsheader">
-<th>Feed</th><th>RSS</th><th>Last update</th><th>Next update</th>
+<th>Feed</th><th>RSS</th><th>Last fetched</th><th>Next fetched after</th>
 </tr>"""
 		feeds = self.feeds.values()
 		feeds.sort(lambda a, b: cmp(a.get_html_name(config).lower(), b.get_html_name(config).lower()))
@@ -1325,6 +1361,8 @@ Report bugs to <ats@offog.org>."""
 def main(argv):
 	"""The command-line interface to the aggregator."""
 
+	locale.setlocale(locale.LC_ALL, "")
+
 	try:
 		(optlist, args) = getopt.getopt(argv, "ulwf:c:tTd:va:N", ["update", "list", "write", "update-feed=", "help", "config=", "show-template", "dir=", "show-itemtemplate", "verbose", "upgrade", "add=", "no-locking"])
 	except getopt.GetoptError, s:
